// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//============================================================================
//  PLL recalibration module
// Contains the reconfiguration master to initiate the PLL recalibration
//============================================================================

`timescale 1 ps / 1 ps

module alt_e40_pll_recal #(
        parameter RX_PLL_TYPE = "IOPLL",
        parameter SIM_EMULATE = 1'b0  
			       )
   (
    input wire 	      mgmt_clk, // managemnt/reconfig clock
    input wire 	      mgmt_reset, // managemnt/reconfig reset
     // PLL control
    input wire 	      cal_busy, // PLL cal busy
    input wire 	      rx_is_lockedtoref, // PLL rx_is_lockedtoref
    input wire        rx_set_locktoref_core, //rx_set_locktoref signal     
    output reg 	      rx_set_locktoref, //rx_set_locktoref signal 
    output reg 	      recal_is_done, // recalibration is finished
     // PLL Reconfig master
    output reg 	      rcfg_write, // AVMM write
    output wire	      rcfg_read, // AVMM read
    output wire [10:0]  rcfg_address, // AVMM address
    output wire [31:0] rcfg_wrdata, // AVMM write data
    input wire [31:0] rcfg_rddata, // AVMM read data
    input wire 	      rcfg_wtrqst    // AVMM wait request
  );

   reg 		      rcfg_read_fpll; // AVMM read
   reg [10:0] 	      rcfg_address_fpll; // AVMM address
   reg [31:0] 	      rcfg_wrdata_fpll; // AVMM write data
   
   assign rcfg_read = (RX_PLL_TYPE == "IOPLL") ? 1'b0 : rcfg_read_fpll;
   assign rcfg_address = (RX_PLL_TYPE == "IOPLL") ? 11'b000_0000_0000 : rcfg_address_fpll;
   assign rcfg_wrdata = (RX_PLL_TYPE == "IOPLL") ? 32'b0000_0000_0000_0000_0000_0000_0000_0000 : rcfg_wrdata_fpll;   

	wire rcfg_wtrqst_mgmt;
    // Synchronize rcfg_wtrqst to mgmt_clk domain
    alt_e40_synchronizer rcfg_wtrqst_sync (
        .clk    (mgmt_clk),
        .din    (rcfg_wtrqst),
        .dout   (rcfg_wtrqst_mgmt)
    );
   
//============================================================================
//  Counter to wait to start the recalibration
//  also has hysteresis in case lock chatters
//  speed up mode for simulation
//============================================================================
`ifdef ALTERA_RESERVED_QIS   // HW mode, full lengh counter
  reg [15:0] cnt_lock;
  reg       cnt_max;

   always @(posedge mgmt_clk or posedge mgmt_reset)
     if (mgmt_reset) begin
	cnt_max  <= 1'b0;
	cnt_lock <= 16'b0;
     end else  begin
	cnt_max <= (cnt_lock == 16'hffff);
	if      (~rx_is_lockedtoref)      cnt_lock <= 10'd0;
	else if (cnt_lock == 16'hffff)  cnt_lock <= cnt_lock;
	else                           cnt_lock <= cnt_lock + 1'b1;
     end
`else     // Sim mode, shortened counter
   reg [2:0] cnt_lock;
   reg       cnt_max;
   
   always @(posedge mgmt_clk or posedge mgmt_reset)
     if (mgmt_reset) begin
	cnt_max  <= 1'b0;
	cnt_lock <= 3'b0;
     end else  begin
	cnt_max <= (cnt_lock == 3'h7);
	if      (~rx_is_lockedtoref)   cnt_lock <= 3'd0;
	else if (cnt_lock == 3'h7)  cnt_lock <= cnt_lock;
	else                        cnt_lock <= cnt_lock + 1'b1;
     end
`endif // QIS 
   
   generate 
   if (RX_PLL_TYPE == "FPLL") begin : FPLL_recal
      //============================================================================
      //  FPLL Recal State Machine
      // CDR PLL output is driving IOPLL or FPLL refclk and clkout of PLL is rx_coreclkin of HSSI
      // in order to come HSSI out of reset properly we need to have proper rx_coreclkinwhich come from pll output
      // and in order to get stable pll output clock we need stable refclk
      // to get stable refclk need is to put CDR in LTR (set_locktoref mode initially
      // and start recal after locktoref is achieved
      // Here is the state machine
      //      wait for locktoref to be active for 10us
      //      write 0x02 to address 0x000 to request IF
      //      read-modify-write bit-1 of address 0x100 to enable Calibration
      //      write 0x03 to address 0x000 to release IF
      //      wait for cal_busy to be deasserted
      //      Done until locktodata deasserted or reset
      //============================================================================
      localparam [2:0] RCL_IDLE    = 3'd0;
      localparam [2:0] RCL_WR2     = 3'd1;
      localparam [2:0] RCL_RDM     = 3'd2;  
      localparam [2:0] RCL_WRM     = 3'd3; 
      localparam [2:0] RCL_WR3     = 3'd4;      
      localparam [2:0] RCL_BSY0    = 3'd5;
      localparam [2:0] RCL_BSY1    = 3'd6;
      localparam [2:0] RCL_DONE    = 3'd7;
      
      reg [2:0]  rcl_state;
      reg [2:0]  rcl_nxt_st;
      
      
      // state register
      always @(posedge mgmt_clk or posedge mgmt_reset)
	if (mgmt_reset)  rcl_state <= RCL_IDLE;
	else             rcl_state <= rcl_nxt_st;
      
      // next state logic
      always @(*) begin
	 rcl_nxt_st = rcl_state;
	 case(rcl_state)
	   RCL_IDLE : 
	     if (cnt_max)       
	       rcl_nxt_st = RCL_WR2;
	   
	   RCL_WR2 :
	     if (~rcfg_wtrqst_mgmt)  
	       rcl_nxt_st = RCL_RDM;
	   RCL_RDM :	     
	     if (~rcfg_wtrqst_mgmt)  
	       rcl_nxt_st = RCL_WRM;

	   RCL_WRM  : 
	     if (~rcfg_wtrqst_mgmt)  
	       rcl_nxt_st = RCL_WR3;

	   RCL_WR3  : 
	     if (~rcfg_wtrqst_mgmt)  
	       rcl_nxt_st = RCL_BSY0;
	   RCL_BSY0 : 
	     if (cal_busy)     
	       rcl_nxt_st = RCL_BSY1;
	   RCL_BSY1 : 
	     if (~cal_busy)     
	       rcl_nxt_st = RCL_DONE;
	   RCL_DONE : 
	     if (~cnt_max)      
	       rcl_nxt_st = RCL_IDLE;
	   default  : rcl_nxt_st = RCL_IDLE;
	 endcase
      end
      
      //============================================================================
      //  Generate DPRIO signals for the PLL reconfig Interface
      //============================================================================
      //recalibration is finished
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   recal_is_done  <= 1'b0;
	 else if(rcl_nxt_st == RCL_DONE)
	   recal_is_done  <= 1'b1;
      end
      
      //DPRIO read
      always  @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   rcfg_read_fpll  <= 1'b0;
	 else if(rcl_nxt_st == RCL_RDM)
	   rcfg_read_fpll  <= 1'b1;
	 else
	   rcfg_read_fpll  <= 1'b0;
      end
      
      //DPRIO write
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   rcfg_write  <= 1'b0;
	 else if((rcl_nxt_st == RCL_WR2) || (rcl_nxt_st == RCL_WRM) || (rcl_nxt_st == RCL_WR3))
	   rcfg_write  <= 1'b1;
	 else
	   rcfg_write  <= 1'b0;
      end
      
      //DPRIO address
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   rcfg_address_fpll <= 11'b0;
	 else if((rcl_nxt_st == RCL_RDM) || (rcl_nxt_st == RCL_WRM))
	   rcfg_address_fpll <= 11'h100;
	 else
	   rcfg_address_fpll <= 11'b0;
      end
      
      //DPRIO writedata
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   rcfg_wrdata_fpll <= 32'b0;
	 else if (rcl_nxt_st == RCL_WR2)
	   rcfg_wrdata_fpll <= 32'h02;
	 else if (rcl_nxt_st == RCL_WR3)
	   rcfg_wrdata_fpll <= 32'h03;
	 else if (rcl_nxt_st == RCL_WRM)
	   rcfg_wrdata_fpll <= rcfg_rddata | 32'h02;
	 else
	   rcfg_wrdata_fpll <= 32'b0;
      end // always_ff @
   end // block: FPLL_recal
   else begin : IOPLL_recal
      //============================================================================
      // IOPLL Recal State Machine
      // CDR PLL output is driving IOPLL or FPLL refclk and clkout of PLL is rx_coreclkin of HSSI
      // In order for HSSI to come out of reset properly, it needs proper rx_coreclkin signal so
      // need stable rx_coreclkin
      // and in order to get stable pll output clock we need stable refclk
      // to get stable refclk need to put CDR in LTR (set_locktoref) mode initially
      // achieve locktoref and then start recal
      // Here is the state machine
      //      wait for locktoref to be active for 20us
      //      give recal request to IOPLL my pulsing mgmt_write for 1 clock cycle 
      //      wait for mgmt_request to be deasserted
      //============================================================================

      
      localparam [1:0] IOPLL_RCL_IDLE    = 2'd0;
      localparam [1:0] IOPLL_RCL_WR     = 2'd1;
      localparam [1:0] IOPLL_RCL_BSY     = 2'd2;
      localparam [1:0] IOPLL_RCL_DONE    = 2'd3;
      
      reg [1:0]  iopll_rcl_state /* synthesis preserve */;
      reg [1:0]  iopll_rcl_nxt_st /* synthesis preserve */;
		  
      // state register
      always @(posedge mgmt_clk or posedge mgmt_reset)
	if (mgmt_reset)  iopll_rcl_state <= IOPLL_RCL_IDLE;
	else             iopll_rcl_state <= iopll_rcl_nxt_st;
      
      // next state logic
      always @(*) 
	begin
	   iopll_rcl_nxt_st = iopll_rcl_state;
	   case(iopll_rcl_state)
	     IOPLL_RCL_IDLE : if (cnt_max)      iopll_rcl_nxt_st = IOPLL_RCL_WR;
	     IOPLL_RCL_WR  : if (~rcfg_wtrqst_mgmt)  iopll_rcl_nxt_st = IOPLL_RCL_BSY;
	     IOPLL_RCL_BSY : if (~rcfg_wtrqst_mgmt)  iopll_rcl_nxt_st = IOPLL_RCL_DONE;	   	   
	     IOPLL_RCL_DONE : if (~cnt_max)     iopll_rcl_nxt_st = IOPLL_RCL_IDLE;
	     default  : iopll_rcl_nxt_st = IOPLL_RCL_IDLE;	   	   
	   endcase // case (IOPLL_rcl_state)
	end
      
      //============================================================================
      //  Generate DPRIO signals for the PLL reconfig Interface
      //============================================================================
      //recalibration is finished
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   recal_is_done  <= 1'b0;
	 else if(iopll_rcl_state == IOPLL_RCL_DONE && !rcfg_wtrqst_mgmt)
	   recal_is_done  <= 1'b1;
      end
      
      //DPRIO write
      always @(posedge mgmt_clk or posedge mgmt_reset) begin
	 if (mgmt_reset)
	   begin
	      rcfg_write  <= 1'b0;
	   end
	 else if(iopll_rcl_state == IOPLL_RCL_WR)
	   rcfg_write  <= 1'b1;
	 else
	   rcfg_write  <= 1'b0;
      end
   end // block: IOPLL_recal
endgenerate

// Return to LTD mode after recal is done
   
   always @(posedge mgmt_clk or posedge mgmt_reset) 
     begin
	if (mgmt_reset)
	  rx_set_locktoref <= 1'b1;
	else
	  begin
	     if (recal_is_done == 1'b1)
	       rx_set_locktoref <= rx_set_locktoref_core;
	  end
     end

   
endmodule // alt_aeu_100_pll_recal

